//Chris Carlson - 220 final project
//KORG NANOKONTROL Midi mapping for 
//interfacing with the looper, spatializer, and sample playback 
//objects

//IMPORTANT: make sure the nanokontrol scene set included with this
//distribution is uploaded to the controller.
//Also - you may need to change the device number below to ensure
//that the proper controller is being used.  Type chuck --probe
//in terminal to get a listing of the midi/audio interface device numbers

//MAPPING DETAILS:
//TRANSPORT:
//record button.  begin/stop recording a loop
//circular arrow button.  hold down and turn channel knob to "select" that channel.  next recording will occur in selected channel.
//(otherwise, loop slots advance by one channel each time the record button is pressed.  after 7th loop, 1st looop will be overwritten)

//stop button - stop current selected channel output
//play current selected channel output
//rewind button - unsolo all soloed chanels
//forward button - stop and clear all loops.  stop all noise samples.

//SCENE 1:  
//Channels 1-7:
//Knob - loop playback rate -1 to 1
//slider - dry gain.  wet gain (when channel 9 bottom button is on)
//top button - turn bidirectional looping on/off
//bottom button - solo channel

//Channel 8:  
//slider:  adc input gain sent to filter/spatializer input

//Channel 9:
//slider:  global dry gain.  global wet gain (when bottom button is on)
//bottom button:  off (all sliders control dry gain).  on ( all sliders control wet gain (to filter/spatializer)
//top button:  global bi directional looping on/off
//knob:  global playback rate (for all loops) -1 to 1

//SCENE 2
//Channels 1 - 8
//slider:  gain of sample
//top button:  pan sample to next audio channel
//bottom button:  pan sample to previous audio channel

//Channel 9
//slider:  gain of all samples (multiplier)




//MIDI Setup
// number of the device to open (see: chuck --probe)
2 => int device;
// get command line
if( me.args() ) me.arg(0) => Std.atoi => device;
// the midi event
MidiIn min;
// the message for retrieving data
MidiMsg msg;
// open the device
if( !min.open( device ) ) me.exit();

// print out device that was opened
<<< "MIDI device:", min.num(), " -> ", min.name() >>>;

//flags/counters for controlling the program flow
0 => int isRecording;
0 => int counter;
0 => int wetCtrl;
0 => int isSelecting;

//get total number of available loop slots
G.looper.getNumLoops() => int numLoops;
while( true )
{   
	while(min.recv(msg))
	{
		//print midi msgs for channel assignment/data monitoring purposes.  comment out during use.
		///<<<"data1" + msg.data1>>>;
		///<<<"data2" + msg.data2>>>;
		///<<<msg.data3>>>;
		
		//if control change message
		if (msg.data1 == 176){
			
			//RECORD
			if(msg.data2 == 44 )
			{
				if (msg.data3 == 127){
					if (isRecording != 1){
						counter % (numLoops -1) => counter;
						//<<<numLoops>>>;
						G.looper.recordNew(counter);
						1 => isRecording;
						<<<"RECORDING TO: " + (counter + 1)>>>; //match slider numbers
					}
				}else{ 
					0 => isRecording;
					G.looper.stopRec(counter);
					///<<<"LOOPING " + counter>>>;
					counter++;
					<<<"NEXT SLOT: " + (counter+ 1)>>>;
				}
			}
			
			
			//DIRECT INPUT
			if(msg.data2 == 12 )
			{
				G.looper.setDirectGain(msg.data3/127.0);
			}
			
			//all on (filter)
			if(msg.data2 == 30)
			{
				if (msg.data3 == 1){
					G.looper.allOn(); //filter spatializer envelopes all on (accessed through looper)
				}
			}
			//all off (filter)
			if(msg.data2 == 39)
			{
				if (msg.data3 == 1){
					G.looper.allOff();//filter spatializer envelopes on/off (through looper)
				}
			}
			
			//channel selector
			if (msg.data2 == 49){
				if (msg.data3 == 127){
					1 => isSelecting;
				}else{
					0 => isSelecting;
				}	
			}
			
			
			//stop loop
			if (msg.data2 == 46){
				if (msg.data3 == 127){
					//<<<"stop loop: " +(counter % (numLoops-1))>>>;
					G.looper.stopLoop(counter % (numLoops -1));
				}
			}
			
			//start loop
			if (msg.data2 == 45){
				if (msg.data3 == 127){
					G.looper.playLoop(counter% (numLoops -1),0.5::second);
				}
			}
			
			
			//stop everything
			if (msg.data2 == 48){
				if (msg.data3 == 127){
					G.noises.allOff();
					G.looper.clearAll();
					0 => counter;
					<<<"NEXT SLOT:  1">>>;
				}
			}
			
			//unsolo all
			if (msg.data2 == 47){
				if (msg.data3 == 127){
					G.looper.unSoloAll();
				}
			}
			
			//GLOBAL CONTROLS (SLOT 9)
			//bidirectional playback 
			if(msg.data2 == 31 )
			{
				G.looper.setBi(msg.data3);
			}
			
			//toggle wet/dry gain control (button)
			if(msg.data2 == 40 )
			{
				msg.data3 => wetCtrl;
			}
			//dry/wet gain global
			if(msg.data2 == 13 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(msg.data3/127.0);
				}else{
					G.looper.send(msg.data3/127.0);
				}
			}
			
			//set global playback rate
			if(msg.data2 == 22 )
			{
				G.looper.setRate(((msg.data3/127.0) -0.5)*2);
			}
			
			
			
			
			///////////////CHANNELS///////////////////////////
			
			
			//////////////LOOP 1 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 23)
			{
				G.looper.setBi(0,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 32 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(0);	
				}else{
					G.looper.unSoloChan(0);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 1 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(0,msg.data3/127.0);
				}else{
					G.looper.send(0,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 14 )
			{
				
				if ((isSelecting == 1) && (isRecording !=1)){
					0 => counter;
				}else{
					
					G.looper.setRate(0,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			
			
			//////////////LOOP 2 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 24)
			{
				G.looper.setBi(1,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 33 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(1);	
				}else{
					G.looper.unSoloChan(1);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 2 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(1,msg.data3/127.0);
				}else{
					G.looper.send(1,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 15 )
			{
				if ((isSelecting == 1) && (isRecording !=1)){
					1 => counter;
				}else{
					
					G.looper.setRate(1,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			
			
			
			//////////////LOOP 3 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 25)
			{
				G.looper.setBi(2,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 34 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(2);	
				}else{
					G.looper.unSoloChan(2);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 3 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(2,msg.data3/127.0);
				}else{
					G.looper.send(2,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 16 )
			{	
				if ((isSelecting == 1) && (isRecording !=1)){
					2 => counter;
				}else{
					
					G.looper.setRate(2,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			
			
			
			
			//////////////LOOP 4 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 26)
			{
				G.looper.setBi(3,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 35 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(3);	
				}else{
					G.looper.unSoloChan(3);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 4 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(3,msg.data3/127.0);
				}else{
					G.looper.send(3,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 17 )
			{	
				if ((isSelecting == 1) && (isRecording !=1)){
					3 => counter;
				}else{
					
					G.looper.setRate(3,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			
			
			
			//////////////LOOP 5 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 27)
			{
				G.looper.setBi(4,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 36 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(4);	
				}else{
					G.looper.unSoloChan(4);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 5 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(4,msg.data3/127.0);
				}else{
					G.looper.send(4,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 18 )
			{
				if ((isSelecting == 1) && (isRecording !=1)){
					4 => counter;
				}else{
					
					G.looper.setRate(4,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			
			
			//////////////LOOP 6 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 28)
			{
				G.looper.setBi(5,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 37 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(5);	
				}else{
					G.looper.unSoloChan(5);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 6 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(5,msg.data3/127.0);
				}else{
					G.looper.send(5,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 19 )
			{
				if ((isSelecting == 1) && (isRecording !=1)){
					5 => counter;
				}else{
					
					G.looper.setRate(5,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			
			//////////////LOOP 7 GROUP//////////////////////
			//bidirectional looping button
			if(msg.data2 == 29)
			{
				G.looper.setBi(6,msg.data3);	
			}
			
			//solo/unsolo button
			if(msg.data2 == 38 )
			{
				if (msg.data3 == 1){
					G.looper.soloChan(6);	
				}else{
					G.looper.unSoloChan(6);
				}
			}
			
			//knob - wet and dry gain depending on mode
			if(msg.data2 == 7 )
			{
				if (wetCtrl == 0){
					G.looper.setGain(6,msg.data3/127.0);
				}else{
					G.looper.send(6,msg.data3/127.0);
				}
			}
			
			//set playback rate
			if(msg.data2 == 20 )
			{
				if ((isSelecting == 1) && (isRecording !=1)){
					6 => counter;
				}else{
					
					G.looper.setRate(6,((msg.data3/127.0) -0.5)*2);
				}
			}
			
			
			//NOISES
			//1
			//gain
			if (msg.data2 == 42){
				G.noises.setGain(0,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 67){
				if (msg.data3 == 1){
					G.noises.pan(0,1);
				}
			}
			if (msg.data2 == 76){
				if (msg.data3 == 1){
					G.noises.pan(0,-1);
				}
			}
			
			//2
			//gain
			if (msg.data2 == 43){
				G.noises.setGain(1,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 68){
				if (msg.data3 == 1){
					
					G.noises.pan(1,1);
				}
			}
			if (msg.data2 == 77){
				if (msg.data3 == 1){
					G.noises.pan(1,-1);
				}
			}
			//3
			//gain
			if (msg.data2 == 50){
				G.noises.setGain(2,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 69){
				if (msg.data3 == 1){
					
					G.noises.pan(2,1);
				}
			}
			if (msg.data2 == 78){
				if (msg.data3 == 1){
					G.noises.pan(2,-1);
				}
			}
			
			//gain
			if (msg.data2 == 51){
				G.noises.setGain(3,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 70){
				if (msg.data3 == 1){
					
					G.noises.pan(3,1);
				}
			}
			if (msg.data2 == 79){
				if (msg.data3 == 1){
					G.noises.pan(3,-1);
				}
			}
			
			//4
			//gain
			if (msg.data2 == 52){
				G.noises.setGain(4,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 71){
				if (msg.data3 == 1){
					G.noises.pan(4,1);
				}
			}
			if (msg.data2 == 80){
				if (msg.data3 == 1){
					G.noises.pan(4,-1);
				}
			}
			
			//5
			//gain
			if (msg.data2 == 53){
				G.noises.setGain(5,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 72){
				if (msg.data3 == 1){
					
					G.noises.pan(5,1);
				}
			}
			if (msg.data2 == 81){
				if (msg.data3 == 1){
					G.noises.pan(5,-1);
				}
			}
			//6
			//gain
			if (msg.data2 == 54){
				G.noises.setGain(6,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 73){
				if (msg.data3 == 1){
					
					G.noises.pan(6,1);
				}
			}
			if (msg.data2 == 82){
				if (msg.data3 == 1){
					G.noises.pan(6,-1);
				}
			}
			
			//7
			//gain
			if (msg.data2 == 55){
				G.noises.setGain(7,msg.data3/127.0);
			}
			//pan f
			if (msg.data2 == 74){
				if (msg.data3 == 1){
					
					G.noises.pan(7,1);
				}
			}
			if (msg.data2 == 83){
				if (msg.data3 == 1){
					G.noises.pan(7,-1);
				}
			}
			
			//global noise gain
			if (msg.data2 == 56){
				G.noises.setGain(msg.data3/127.0);
			}
			
			
			
		}		
	}
	
	//advance time
	5::ms => now;
}